/* ****************************************************************************
  Configurable little countdown with binary display ot the counter.

  Franois Pessaux (c) 2015
  This piece of code can be freely modified and/or modified.
  *************************************************************************** */

/*
#include <arduino.h>
Add the above line to compile with a Makefile. Leave it commented to compile
with the Arduino environment.
*/

#include "wiring_config.h"
#include "button.h"
#include "leds.h"
#include "timers.h"

/* Channels used for the timer 1.
   Channel 0 to count down by 1 each second.
   Channel 1 to count the time left to the user to enter the countdown value. */
#define COUNT_CHANNEL (0)
#define SETUP_CHANNEL (1)



/* Values to initialize the timers. */
#define COUNT_PERIOD (1.0)                                       /* Seconds. */
#define COUNT_FREQ (1.0 / COUNT_PERIOD)                          /* Hz. */
#define COUNT_TIMER_128_RC ((VARIANT_MCK) / 128 / (COUNT_FREQ))  /* Ticks. */

#define SETUP_PERIOD (4.0)                                       /* Seconds. */
#define SETUP_FREQ (1.0 / SETUP_PERIOD)                          /* Hz. */
#define SETUP_TIMER_128_RC ((VARIANT_MCK) / 128 / (SETUP_FREQ))  /* Ticks. */



/** States of the automaton.

    ---------        ----------        --------------        -------------
   | AS_Idle | ---> | AS_Setup | ---> | AS_InitCount | ---> | AS_Counting | -
    ---------        ----------        --------------        -------------   |
        ^                                                                    |
       / \                                                                   |
        |                                                                    |
        ---------------------------------------------------------------------
 */
enum automaton_state_r { AS_Idle, AS_Setup, AS_InitCount, AS_Counting } ;
enum automaton_state_r state = AS_Idle ;



/** Value of the countdown. It is initially set to the value entered by the
    user. Then it gets decremented each second until it reaches 0. */
volatile unsigned short count_down ;



/** Interrupt handler for the timer firing each second and lightling the LEDs
    according to the value of the countdown and decrements the counter. */
void TC3_Handler ()
{
  count_down = (count_down - 1) % 8 ;
  LED_of_int (count_down) ;
  /* Get the status to clear it and allow the interrupt to fire again. */
  TC_GetStatus (TC1, COUNT_CHANNEL) ;
}



/** Interrupt handler for the timer firing once the time given to the user
    to indicate the value of the countdown is expired.
    It stops its own timer and set the automaton in the state
    "start counting." */
void TC4_Handler ()
{
  /* Get the status to clear it and allow the interrupt to fire again. */
  TC_GetStatus (TC1, SETUP_CHANNEL) ;
  /* ... But stop the timer as soon. */
  stop_timer_TC1 (SETUP_CHANNEL) ;
  state = AS_InitCount ;
}



/** Setup routine. Mostly configure the 5 used pins and clear all the LEDs. */
void setup ()
{
  /* Three LEDs in output. */
  pinMode (LED2_PIN, OUTPUT) ;
  pinMode (LED1_PIN, OUTPUT) ;
  pinMode (LED0_PIN, OUTPUT) ;
  /* Two switches in input. */
  pinMode (SW_CONFIG_PIN, INPUT) ;
  pinMode (SW_SETCOUNT_PIN, INPUT) ;
  /* Turn off the 3 LEDs. */
  LED_of_int (0) ;
}



/** The mainloop called forever.
    Here is the transition function of the automaton handling the 4 states
    the machine can be in:
    - AS_Idle: nothing to do, the previous countdown is ended, the user didn't
        press the counter setup button.
    - AS_Setup: the user pressed the counter setup button and will press
        the other button to indicate the number of ticks the countdown will
        have to perform. The setup delay starts to elapse.
    - AS_InitCount: the delay left to the user to set the number of ticks the
        countdown must perform is ended, the countdown must start.
    - AS_Counting: the countdown is running and will end ass soon as 0 is
        reached.
*/
void loop ()
{
  switch (state) {
  case AS_Idle:
    /* Wait for a press on the counter setup button. No matter the noise since
       once a HIGH level is detected, the automaton will change its state,
       hence will ignore following LOW-HIGH noise. */
    if (digitalRead (SW_CONFIG_PIN) == HIGH) {
      count_down = 0 ;         /* Reset the countdown limit. */
      start_timer_TC1 (SETUP_TIMER_128_RC, SETUP_CHANNEL) ;
      state = AS_Setup ;   /* Go in state "setup". */
    }
    break ;

  case AS_Setup:
    /* Don't use:
       if (digitalRead (SW_SETCOUNT_PIN) == HIGH)
       which does not work due to pooling speed and noise in the signal. */
    if (button_clicked (SW_SETCOUNT_PIN)) {
      count_down ++ ;  /* Each press means one more to countdown. */
      if (count_down > 7) count_down = 7 ;  /* Limit to the 3 LEDS we have. */
    }
    break ;

  case AS_InitCount:
    /* Turn the LEDs to the initial value of the counter. */
    LED_of_int (count_down) ;
    /* Launch the timer firing each second. */
    start_timer_TC1 (COUNT_TIMER_128_RC, COUNT_CHANNEL) ;
    state = AS_Counting ;   /* Go in state "counting". */
    break ;

  case AS_Counting:
    if (count_down == 0) {
      stop_timer_TC1 (COUNT_CHANNEL) ;
      state = AS_Idle ;
    }
    break ;
  }
}
